package argonavis.rtf;

import java.io.*;

import org.apache.tools.ant.*;
import org.xml.sax.InputSource;
import org.jfor.jfor.converter.Converter;

/**
 * This Ant task uses Apache FOP 0.20 (xml.apache.org) to
 * transform XSL-FO documents into PDF documents
 */
public class RtfTask extends Task {
    
    private File sourceDir, destDir;
    private String includes = ".fo";
    
    /**
     * For each FO file in sourceDir, generate a
     * RTF file and store it in destDir.
     * If RTF generation fails, skip to next file.
     * If all transformations fail, the task should fail.
     */
    public void execute() throws BuildException {
        String[] fileNames = sourceDir.list(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                if(name.indexOf(includes) != -1) {
                    return true;
                }
                return false;
            }
        });
        if (fileNames == null || fileNames.length == 0) {
            throw new BuildException("No files to transform!");
        }
        boolean oneWorked = false;
        for (int i = 0; i < fileNames.length; i++) {
            System.out.println("Processing " + fileNames[i]);
            int extDot = fileNames[i].lastIndexOf(".");
            String rootName = fileNames[i].substring(0, extDot);
            try {
                generateRTF(new File(sourceDir, fileNames[i]), new File(destDir, rootName + ".rtf"));
                oneWorked = true;
            } catch (IOException e) {
                System.out.println("Generation of " + rootName + ".rtf from "+fileNames[i]+" failed!");
                continue;
            }
        }
        if (!oneWorked) { 
            throw new BuildException("No RTF files were generated!");
        }
    }
    
    /**
     * Defines the basedir attribute
     */
    public void setBasedir(File sourceDir) throws BuildException {
        if (!sourceDir.isDirectory() || !sourceDir.exists()) {
            throw new BuildException("Source directory " + sourceDir + " does not exist!");
        }
        this.sourceDir = sourceDir;
    }
    
    /**
     * Defines the destdir attribute
     */
    public void setDestdir(File destDir) throws BuildException {
        if (!destDir.isDirectory() || !destDir.exists()) {
            throw new BuildException("Destination directory " + destDir + " does not exist!");
        }
        this.destDir = destDir;
    }
    
    /**
     * This method generates a RTF file from a XSL-FO styled file
     * using JFOR (www.jfor.org) libraries.
     */
    public static void generateRTF(File foFile, File rtfFile) throws IOException {
        FileOutputStream out = new FileOutputStream(rtfFile);
        InputSource in = new InputSource(new FileInputStream(foFile));
        try {
		    new Converter(in, new OutputStreamWriter(out), Converter.createConverterOption ());
        } catch (Exception e) { 
            if (e instanceof RuntimeException) {
                throw (RuntimeException)e;
            }
            throw new IOException("Unable to run RTF converter! " + e);
        }
		
    }
    
    /**
     * Only for testing. Package-level access.
     */
    File getBasedir() { return sourceDir; }
    /**
     * Only for testing. Package-level access.
     */
    File getDestdir() { return destDir; }
    
}
